require 'aws-sdk-v1'

module Cloudutil
  module AWS
    # Class containing common methods and attributes for interfacing with AWS
    class Core
      DEFAULT_REGION = 'us-east-1'

      attr_writer :region
      attr_writer :access_key
      attr_writer :secret_key

      # Initialize an instance of this class.  Will accept options as a hash, or an array,
      # with hash form being preferred.  There are no required parameters for this method.
      #
      # Array args must be in the order of: AWS region, AWS access key, AWS secret key.
      #
      # Supported hash option keys are:
      #  - config (an pre-built AWS::config object to use for API calls)
      #  - region (the AWS region to make API calls to)
      #  - access_key (the AWS access key to use for API authentication)
      #  - secret_key (the AWS secret key to use for API authentication)
      #
      # If no AWS::config object was passed, this class's config method will automatically
      # be called to create one
      #
      def initialize(*args)
        if args.is_a? Hash
          _process_hash_init(args.to_h)
        elsif args[0].respond_to? :to_h
          _process_hash_init(args[0].to_h)
        else
          _process_array_init(args.flatten)
        end

        config(true) if @config.nil?
      end

      # Getter for the 'region' attribute, uses DEFAULT_REGION
      # if not explicitly set
      def region
        @region = DEFAULT_REGION if !@region || @region.length < 1
        @region
      end

      # Getter for the 'access_key' attribute, will use values of the
      # AWS_ACCESS_KEY_ID or AWS_ACCESS_KEY env vars, if not explicitly set
      def access_key
        if !@access_key || @access_key.length < 1
          @access_key = ENV['AWS_ACCESS_KEY_ID'] || ENV['AWS_ACCESS_KEY'] || nil
        end

        @access_key
      end

      # Getter for the 'secret_key' attribute, will use values of the
      # AWS_SECRET_ACCESS_KEY or AWS_SECRET_KEY env vars, if not explicitly set
      def secret_key
        if !@secret_key || @secret_key.length < 1
          @secret_key = ENV['AWS_SECRET_ACCESS_KEY'] || ENV['AWS_SECRET_KEY'] || nil
        end

        @secret_key
      end

      # Create (if not already existing) and return an AWS::config object.
      # If creating a new AWS::config object, will make calls to this class's
      # region, access_key, and secret_key methods to populate the config data
      def config(reconfig = false)
        if !@config || reconfig
          @config = ::AWS.config(region: region, access_key_id: access_key, secret_access_key: secret_key)
        end

        @config
      end

      private

      def _process_hash_init(args)
        return unless args
        @config = args[:config]
        @region = args[:region]
        @access_key = args[:access_key]
        @secret_key = args[:secret_key]
      end

      def _process_array_init(args)
        _process_hash_init(region: args[0], access_key: args[1], secret_key: args[2]) if args
      end
    end
  end
end
