require_relative './core'

module Cloudutil
  module AWS
    # A class of utility methods for the AWS EC2 service
    class EC2 < Cloudutil::AWS::Core
      # Resolve the provided value to an AWS subnet id.  By default, the search is not case-sensitive,
      # but that can be overridden by supplying 'false' as a second argument to this method.
      # Will return the 1st value in a set, if multiple values returned.
      def resolve_subnet_id(subnet, ignore_case = true)
        obj = lookup_subnet(subnet, ignore_case).first
        return obj.subnet_id unless obj.nil?
        nil
      end

      # Resolve the provided value to an array of AWS subnet objects.  By default, the search is not
      # case-sensitive, but that can be overridden by supplying 'false' as a second argument to this method.
      def lookup_subnet(subnet, ignore_case = true, nap = 2, retries = 5)
        ec2 = ::AWS::EC2.new
        subnets = []

        begin
          if aws_subnet_id? subnet
            subnets << ec2.subnets[subnet]
          else
            subnets = do_subnet_lookup(ec2.subnets.filter('state', 'available'), subnet, ignore_case)
          end
        rescue ::AWS::EC2::Errors::RequestLimitExceeded, ::AWS::Errors::ServerError
          raise if retries < 1
          sleep nap
          lookup_subnet(subnet, ignore_case, (nap * 2), (retries - 1))
        end

        subnets
      end

      # Returns true if the supplied value looks like an AWS subnet id
      def aws_subnet_id?(subnet)
        return true if subnet.match(/^subnet-\h{8,}$/)
        false
      end

      # Resolve the provided value to an AWS sec group id.  By default, the search is not case-sensitive,
      # but that can be overridden by supplying 'false' as a second argument to this method.
      # Will return the 1st value in a set, if multiple values returned.
      def resolve_security_group_id(group, ignore_case = true)
        obj = lookup_security_group(group, ignore_case).first
        return obj.security_group_id unless obj.nil?
        nil
      end

      # Resolve the provided value to an array of AWS sec group objects.  By default, the search is not
      # case-sensitive, but that can be overridden by supplying 'false' as a second argument to this method.
      def lookup_security_group(group, ignore_case = true, nap = 2, retries = 5)
        ec2 = ::AWS::EC2.new
        groups = []

        begin
          if aws_security_group_id? group
            groups << ec2.security_groups[group]
          else
            groups = do_security_group_lookup_by_name(ec2, group, ignore_case)
            groups = do_security_group_lookup_by_name_tag(ec2, group, ignore_case) if groups.nil? || groups.empty?
          end
        rescue ::AWS::EC2::Errors::RequestLimitExceeded, ::AWS::Errors::ServerError
          raise if retries < 1
          sleep nap
          lookup_security_group(group, ignore_case, (nap * 2), (retries - 1))
        end

        groups
      end

      # Returns true if the supplied value looks like an AWS security group id
      def aws_security_group_id?(group)
        return true if group.match(/^sg-\h{8,}$/)
        false
      end

      # Resolve the provided value to an AWS AMI id.  By default, the search is not case-sensitive,
      # but that can be overridden by supplying 'false' as a second argument to this method.
      # Will return the 1st value in a set, if multiple values returned.
      def resolve_ami_id(ami, ignore_case = true)
        obj = lookup_ami(ami, ignore_case).first
        return obj.image_id unless obj.nil?
        nil
      end

      # Resolve the provided value to an array of AWS AMI objects.  By default, the search is not
      # case-sensitive, but that can be overridden by supplying 'false' as a second argument to this method.
      def lookup_ami(ami, ignore_case = true, nap = 2, retries = 5)
        ec2  = ::AWS::EC2.new
        amis = []

        begin
          if aws_ami_id? ami
            amis << ec2.images[ami]
          else
            # Don't bother memoizing, this is fast
            imgs = ec2.images.filter('image-type', 'machine').filter('state', 'available')
            amis = lookup_amis(imgs, ami, ignore_case)
          end
        rescue ::AWS::EC2::Errors::RequestLimitExceeded, ::AWS::Errors::ServerError
          raise if retries < 1
          sleep nap
          lookup_ami(ami, ignore_case, (nap * 2), (retries - 1))
        end

        amis
      end

      # Returns true if the supplied value looks like an AWS AMI id
      def aws_ami_id?(ami)
        return true if ami.match(/^ami-\h{8,}$/)
        false
      end

      protected

      def lookup_amis(filter, ami, ignore_case)
        amis = do_ami_lookup_by_name_tag(filter, ami, ignore_case)

        if amis.nil? || amis.empty
          amis = do_private_ami_lookup_by_name(filter, ami, ignore_case)
          amis = do_ami_lookup_by_name(filter, ami, ignore_case) if amis.nil? || amis.empty?
        end

        amis
      end

      # Find an AMI based on the value of the 'Name' tag
      def do_ami_lookup_by_name_tag(filter, ami, ignore_case)
        # Using memoization, this is fast, relativly speaking (can be < 1 sec, 15-20 sec w/o memoization)
        ::AWS.memoize do
          filter.tagged('Name').select do |obj|
            !obj.tags.Name.nil? &&
              if ignore_case
                obj.tags.Name.casecmp(ami) == 0
              else
                obj.tags.Name.eql?(ami)
              end
          end
        end
      end

      # Find a non-public AMI based on the value of the AMI's name property
      def do_private_ami_lookup_by_name(filter, ami, ignore_case)
        # Do lookup of non-public AMIs we have access to.  This should be a really short list,
        # so it should come back pretty fast
        ::AWS.memoize do
          if ignore_case
            filter.filter('is-public', 'false')
              .select { |obj| !obj.name.nil? && obj.name.casecmp(ami) == 0 }
          else
            filter.filter('is-public', 'false')
              .select { |obj| !obj.name.nil? && obj.name.eql?(ami) }
          end
        end
      end

      # Find an AMI based on the value of the AMI's name property
      def do_ami_lookup_by_name(filter, ami, ignore_case)
        # This is an absolute pig of a lookup (30+ sec to return), that's why we do it after the faster tag
        # and private AMI lookups. Otherwise this code ends up inspecting all 31k+ public AMIs on AWS
        ::AWS.memoize do
          if ignore_case
            filter.select { |obj| !obj.name.nil? && obj.name.casecmp(ami) == 0 }
          else
            filter.select { |obj| !obj.name.nil? && obj.name.eql?(ami) }
          end
        end
      end

      # Find a VPC subnet based on the 'Name' tag (subnets don't have a name attribute)
      def do_subnet_lookup(filter, subnet, ignore_case)
        ::AWS.memoize do
          filter.tagged('Name').select do |obj|
            !obj.tags.Name.nil? &&
              if ignore_case
                obj.tags.Name.casecmp(subnet) == 0
              else
                obj.tags.Name.eql?(subnet)
              end
          end
        end
      end

      # Find a VPC security group based on the name attribute
      def do_security_group_lookup_by_name(ec2, group, ignore_case)
        ::AWS.memoize do
          if ignore_case
            ec2.security_groups.select { |obj| !obj.name.nil? && obj.name.casecmp(group) == 0 }
          else
            ec2.security_groups.select { |obj| !obj.name.nil? && obj.name.eql?(group) }
          end
        end
      end

      # Find a VPC security group based on the 'Name' tag
      def do_security_group_lookup_by_name_tag(ec2, group, ignore_case)
        ::AWS.memoize do
          ec2.security_groups.tagged('Name').select do |obj|
            !obj.tags.Name.nil? &&
              if ignore_case
                obj.tags.Name.casecmp(group) == 0
              else
                obj.tags.Name.eql?(group)
              end
          end
        end
      end
    end
  end
end
